@@ -4,6 +4,9 @@
 /* */
 #undef VERSION
 
+/* */
+#undef HAVE_OPENMP
+
 /* */
 #undef HAVE_TYPEOF
 
@@ -64,6 +67,9 @@
 /* */
 #undef HAVE_STRCSPN
 
+/* */
+#undef HAVE_STRSPN
+
 /* */
 #undef HAVE_STRSTR
 
@@ -193,5 +199,8 @@
 /* Tiff by Brain */
 #undef HAVE_TIFF
 
+/* SVG */
+#undef HAVE_SVG
+
 /* Gpm_Event has wdx and wdy */
 #undef HAVE_WDX_WDY
