@@ -15,23 +15,23 @@ static struct list_description blocks_ld={
 	&blocks,	/*list head*/
 	block_new_item, /*ext_new_item,*/
 	block_edit_item, /*ext_edit_item,*/
-	assoc_default_value, /*assoc_default_value,*/
+	NULL, /*default_value,*/
 	block_delete_item, /*ext_delete_item,*/
 	block_copy_item, /*ext_copy_item,*/
 	block_type_item, /*ext_type_item,*/
 	block_find_item, /*ext_find_item,*/
 	&block_search_histroy, /*&ext_search_history,*/
 	0,		/* this is set in init_assoc function */
-	40,  /* width of main window */
 	15,  /* # of items in main window */
-	T_BLOCK_LIST, /*item title*/
+	T_BLOCKED_IMAGE, /*item title*/
 	T_BLOCK_LIST_IN_USE, /*Already open message*/
 	T_BLOCK_LIST_MANAGER, /*Window title*/
 	T_BLOCK_DELETE,
 	0,	/* no button */
 	NULL,	/* no button */
+	NULL,	/* no save */
 
-	0,0,0,0,  /* internal vars */
+	NULL,NULL,0,0,  /* internal vars */
 	0, /* modified */
 	NULL,
 	NULL,
@@ -42,11 +42,11 @@ static struct list_description blocks_ld={
 static void* block_new_item(void* ignore)
 {
 	/*Default constructor*/
-	struct block *new; 
+	struct block *neww;
 
-	if (!(new = mem_alloc(sizeof(struct block)))) return NULL;
-	new->url = stracpy("");
-	return new;
+	neww = mem_alloc(sizeof(struct block));
+	neww->url = stracpy(cast_uchar "");
+	return neww;
 }
 
 static void block_delete_item(void* data)
@@ -82,19 +82,18 @@ static unsigned char *block_type_item(struct terminal *term, void *data, int x)
 
 	if ((struct list*)item==(&blocks)) return stracpy(_(TEXT_(T_BLOCK_LIST),term));
 	txt=stracpy(item->url);
-	
 
 	/*I have no idea what this does, but it os copied from working code in types.c*/
-	table=get_translation_table(blocks_ld.codepage,term->spec->charset);
-	txt1=convert_string(table,txt,strlen(txt),NULL);
+	table=get_translation_table(blocks_ld.codepage,term_charset(term));
+	txt1=convert_string(table,txt,(int)strlen(cast_const_char txt),NULL);
 	mem_free(txt);
-			
+
 	return txt1;
 }
 
 struct assoc_ok_struct{
 	void (*fn)(struct dialog_data *,void *,void *,struct list_description *);
-	void *data;	
+	void *data;
 	struct dialog_data *dlg;
 };
 
@@ -121,28 +120,29 @@ static void block_edit_done(void *data)
 
 	/*See block_edit_item*/
 	url=(unsigned char *)&d->items[4];
-	
-	table=get_translation_table(s->dlg->win->term->spec->charset,blocks_ld.codepage);
-	txt=convert_string(table,url,strlen(url),NULL);
+
+	table=get_translation_table(term_charset(s->dlg->win->term),blocks_ld.codepage);
+	txt=convert_string(table,url,(int)strlen(cast_const_char url),NULL);
 	mem_free(item->url); item->url=txt;
 
 	s->fn(s->dlg,s->data,item,&blocks_ld);
-	d->udata=0;  /* for abort function */
+	d->udata=NULL;  /* for abort function */
 }
 
 static void block_edit_item_fn(struct dialog_data *dlg)
-{	
+{
 	/*Copied from input_field. I don't know how most of it works.*/
 #define LL gf_val(1, G_BFU_FONT_SIZE)
 	struct terminal *term = dlg->win->term;
 	int max = 0, min = 0;
 	int w, rw;
 	int y = gf_val(-1, -G_BFU_FONT_SIZE);
+	unsigned char *text = TEXT_(T_ENTER_URL);
 
 
 	if (dlg->win->term->spec->braille) y += gf_val(1, G_BFU_FONT_SIZE);
-	max_text_width(term, dlg->dlg->udata, &max, AL_LEFT);
-	min_text_width(term, dlg->dlg->udata, &min, AL_LEFT);
+	max_text_width(term, text, &max, AL_LEFT);
+	min_text_width(term, text, &min, AL_LEFT);
 	max_buttons_width(term, dlg->items + 1, 2, &max);
 	min_buttons_width(term, dlg->items + 1, 2, &min);
 	if (max < dlg->dlg->items->dlen) max = dlg->dlg->items->dlen;
@@ -150,7 +150,7 @@ static void block_edit_item_fn(struct dialog_data *dlg)
 	if (w > max) w = max;
 	if (w < min) w = min;
 	rw = w;
-	dlg_format_text_and_field(dlg, NULL, dlg->dlg->udata, dlg->items, 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
+	dlg_format_text_and_field(dlg, NULL, text, dlg->items, 0, &y, w, &rw, COLOR_DIALOG_TEXT, AL_LEFT);
 	y += LL;
 	dlg_format_buttons(dlg, NULL, dlg->items + 1, 2, 0, &y, w, &rw, AL_CENTER);
 	w = rw;
@@ -160,7 +160,7 @@ static void block_edit_item_fn(struct dialog_data *dlg)
 	draw_dlg(dlg);
 	y = dlg->y + DIALOG_TB;
 	if (dlg->win->term->spec->braille) y += gf_val(1, G_BFU_FONT_SIZE);
-	dlg_format_text_and_field(dlg, term, dlg->dlg->udata, dlg->items, dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
+	dlg_format_text_and_field(dlg, term, text, dlg->items, dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
 	y += LL;
 	dlg_format_buttons(dlg, term, dlg->items + 1, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_CENTER);
 }
@@ -170,7 +170,7 @@ static void block_edit_item(struct dialog_data *dlg, void *data, void (*ok_fn)(s
 {
 	/*Copied from types.c */
 	/*Data is a new item generated by the "default" function*/
-	struct block *new=(struct block*)data;
+	struct block *neww=(struct block*)data;
 
 	struct terminal *term=dlg->win->term;
 	struct dialog *d;
@@ -179,24 +179,20 @@ static void block_edit_item(struct dialog_data *dlg, void *data, void (*ok_fn)(s
 
 
 	/*Allocate space for dialog, 4 items followed by 1 string*/
-	if (!(d = mem_alloc(sizeof(struct dialog) + 4 * sizeof(struct dialog_item) + 1 * MAX_STR_LEN))) return;
+	d = mem_alloc(sizeof(struct dialog) + 4 * sizeof(struct dialog_item) + 1 * MAX_STR_LEN);
 	memset(d, 0, sizeof(struct dialog) + 4 * sizeof(struct dialog_item) + 1 * MAX_STR_LEN);
 
 	/*Set up this string */
 	url=(unsigned char *)&d->items[4];
-	if (new->url)safe_strncpy(url,new->url,MAX_STR_LEN);
-	
+	if (neww->url)safe_strncpy(url,neww->url,MAX_STR_LEN);
+
 	/* Create the dialog */
-	if (!(s=mem_alloc(sizeof(struct assoc_ok_struct))))
-	{
-		mem_free(d);
-		return;
-	}
+	s = mem_alloc(sizeof(struct assoc_ok_struct));
 
 	s->fn=ok_fn;
 	s->data=ok_arg;
 	s->dlg=dlg;
-		
+
 	switch (dlg_title)
 	{
 		case TITLE_EDIT:
@@ -253,7 +249,7 @@ static void *block_find_item(void *start, unsigned char *str, int direction)
 				if (e->url && casestrstr(e->url,str)) return e;
 			}
 	}
-	
+
 	if (e==s&&e->depth>-1&&e->url && casestrstr(e->url,str)) return e;
 
 	return NULL;
@@ -266,39 +262,33 @@ void block_manager(struct terminal *term,void *fcp,struct session *ses)
 }
 
 
-void* block_add_URL_fn(void* garbage, unsigned char* url)
+void *block_url_add(struct session *ses, unsigned char *url)
 {
 	/*Callback from the dialog box created from the link menu*/
-	struct block* new_b, *here;
+	struct block* new_b;
+	struct terminal *term = ses ? ses->term : NULL;
+
+	if (test_list_window_in_use(&blocks_ld, term))
+		return NULL;
 
 	new_b = block_new_item(0);
 
-	if(!new_b)
-		return 0;
-	if(new_b->url) 
+	if (new_b->url)
 		mem_free(new_b->url);
-	
+
 	new_b->url = stracpy(url);
 	new_b->type = 0;
-	
-	here = (struct block*)next_in_tree(&blocks_ld, &blocks);
 
-	add_at_pos(here, new_b);
-	return 0;
+	add_at_pos((struct block *)blocks_ld.list->prev, new_b);
+	return NULL;
 }
 
-void block_add_URL(struct terminal *term, void *xxx, struct session *ses)
+void block_url_query(struct session *ses, unsigned char *u)
 {
-	/*"Block Image" menu item calls this function*/
-	unsigned char *u;
-	struct f_data_c *fd = current_frame(ses);
+	if (test_list_window_in_use(&blocks_ld, ses->term))
+		return;
 
-	if (!fd) return;
-	if (fd->vs->current_link == -1) return;
-	if (!(u = fd->f_data->links[fd->vs->current_link].where_img)) return;
-
-
-	input_field(ses->term, NULL, TEXT_(T_BLOCK_URL) , TEXT_(T_BLOCK_ADD), ses, 0, MAX_INPUT_URL_LEN, u, 0, 0, NULL, TEXT_(T_OK), (void (*)(void *, unsigned char *)) block_add_URL_fn, TEXT_(T_CANCEL), NULL, NULL);
+	input_field(ses->term, NULL, TEXT_(T_BLOCK_URL) , TEXT_(T_BLOCK_ADD), ses, 0, MAX_INPUT_URL_LEN, u, 0, 0, NULL, TEXT_(T_OK), (void (*)(void *, unsigned char *)) block_url_add, TEXT_(T_CANCEL), NULL, NULL);
 
 }
 
@@ -327,7 +317,7 @@ static int simple_glob_match(unsigned char *s, unsigned char *p)
 		p += i + 1;
 		if (!(s = find_first_match(s, p, &i))) return 0;
 		if (!p[i]) {
-			s += strlen(s) - i;
+			s += strlen(cast_const_char s) - i;
 			return !!find_first_match(s, p, &i);
 		}
 	}
@@ -336,7 +326,7 @@ static int simple_glob_match(unsigned char *s, unsigned char *p)
 
 int is_url_blocked(unsigned char* url)
 {
-	struct block* b;	
+	struct block* b;
 
 	foreach(b, blocks)
 	{
@@ -358,7 +348,7 @@ void free_blocks(void)
 	}
 
 	free_list(blocks);
-	free_list(block_search_histroy.items);	
+	free_list(block_search_histroy.items);
 }
 
 
