@@ -13,9 +13,9 @@ struct table_entry {
 };
 
 struct codepage_desc {
-	unsigned char *name;
-	unsigned char **aliases;
-	struct table_entry *table;
+	const char *name;
+	const char * const *aliases;
+	const struct table_entry *table;
 };
 
 #include "codepage.inc"
@@ -23,7 +23,7 @@ struct codepage_desc {
 #include "entity.inc"
 #include "upcase.inc"
 
-static unsigned char strings[256][2] = {
+static const unsigned char strings[256][2] = {
 	"\000", "\001", "\002", "\003", "\004", "\005", "\006", "\007",
 	"\010", "\011", "\012", "\013", "\014", "\015", "\016", "\017",
 	"\020", "\021", "\022", "\023", "\024", "\025", "\026", "\033",
@@ -65,17 +65,17 @@ static void free_translation_table(struct conv_table *p)
 	mem_free(p);
 }
 
-static unsigned char *no_str = "*";
+static const unsigned char no_str[] = "*";
 
 static void new_translation_table(struct conv_table *p)
 {
 	int i;
 	for (i = 0; i < 256; i++) if (p[i].t) free_translation_table(p[i].u.tbl);
-	for (i = 0; i < 128; i++) p[i].t = 0, p[i].u.str = strings[i];
-	for (; i < 256; i++) p[i].t = 0, p[i].u.str = no_str;
+	for (i = 0; i < 128; i++) p[i].t = 0, p[i].u.str = cast_uchar strings[i];
+	for (; i < 256; i++) p[i].t = 0, p[i].u.str = cast_uchar no_str;
 }
 
-static int strange_chars[32] = {
+static const int strange_chars[32] = {
 	0x20ac, 0x0000, 0x002a, 0x0000, 0x201e, 0x2026, 0x2020, 0x2021,
 	0x005e, 0x2030, 0x0160, 0x003c, 0x0152, 0x0000, 0x0000, 0x0000,
 	0x0000, 0x0060, 0x0027, 0x0022, 0x0022, 0x002a, 0x2013, 0x2014,
@@ -89,10 +89,10 @@ unsigned char *u2cp(int u, int to, int fallback)
 {
 	int j, s;
 	again:
-	if (u < 128) return strings[u];
-	if (u == 0xa0) return "\001";
-	if (u == 0xad) return "";
-	if (codepages[to].table == table_utf_8) return encode_utf_8(u);
+	if (u < 128) return cast_uchar strings[u];
+	if (u == 0xa0) return cast_uchar strings[1];
+	if (u == 0xad) return cast_uchar strings[0];
+	if (to == utf8_table) return encode_utf_8(u);
 	if (u < 0xa0) {
 		u = strange_chars[u - 0x80];
 		if (!u) return NULL;
@@ -100,16 +100,16 @@ unsigned char *u2cp(int u, int to, int fallback)
 	}
 	for (j = 0; codepages[to].table[j].c; j++)
 		if (codepages[to].table[j].u == u)
-			return strings[codepages[to].table[j].c];
+			return cast_uchar strings[codepages[to].table[j].c];
 	if (!fallback) return NULL;
 	BIN_SEARCH(N_UNICODE_7B, U_EQUAL, U_ABOVE, u, s);
-	if (s != -1) return unicode_7b[s].s;
+	if (s != -1) return cast_uchar unicode_7b[s].s;
 	return NULL;
 }
 
 int cp2u(unsigned ch, int from)
 {
-	struct table_entry *e;
+	const struct table_entry *e;
 	if (from == utf8_table) return ch;
 	if (from < 0 || ch < 0x80) return ch;
 	for (e = codepages[from].table; e->c; e++) if (e->c == ch) return e->u;
@@ -121,7 +121,7 @@ static unsigned char utf_buffer[7];
 unsigned char *encode_utf_8(int u)
 {
 	memset(utf_buffer, 0, 7);
-	if (u < 0x80) utf_buffer[0] = u;
+	if (u < 0x80) utf_buffer[0] = (unsigned char)u;
 	else if (u < 0x800)
 		utf_buffer[0] = 0xc0 | ((u >> 6) & 0x1f),
 		utf_buffer[1] = 0x80 | (u & 0x3f);
@@ -191,10 +191,11 @@ static struct conv_table *get_translation_table_to_utf_8(int from)
 	static int lfr = -1;
 	if (from == -1) return NULL;
 	if (from == lfr) return utf_table;
+	lfr = from;
 	if (utf_table_init) memset(utf_table, 0, sizeof(struct conv_table) * 256), utf_table_init = 0;
 	else free_utf_table();
-	for (i = 0; i < 128; i++) utf_table[i].u.str = strings[i];
-	if (codepages[from].table == table_utf_8) {
+	for (i = 0; i < 128; i++) utf_table[i].u.str = cast_uchar strings[i];
+	if (from == utf8_table) {
 		for (i = 128; i < 256; i++) utf_table[i].u.str = stracpy(strings[i]);
 		return utf_table;
 	}
@@ -245,7 +246,7 @@ unsigned short int utf8_2_uni_table[0x200] = {
 };
 
 unsigned char utf_8_1[256] = {
-	0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
+	6, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
 	7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
 	7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
 	7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
@@ -253,33 +254,40 @@ unsigned char utf_8_1[256] = {
 	7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
 	7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
 	7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-	5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
+	6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
+	6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
+	6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
+	6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
+	6, 6, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
 	5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
 	4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
-	3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 2, 2, 1, 1, 0, 0,
+	3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 2, 2, 1, 1, 6, 6,
 };
 
-int get_utf_8(unsigned char **s)
+static const unsigned min_utf_8[9] = {
+	0, 0x4000000, 0x200000, 0x10000, 0x800, 0x80, 0x100, 0x1,
+};
+
+unsigned get_utf_8(unsigned char **s)
 {
-	int v, l;
+	unsigned v, min;
+	int l;
 	unsigned char *p = *s;
-	if (!(l = utf_8_1[p[0]])) {
-		(*s)++;
-		ret:
-		return 0;
-	}
+	l = utf_8_1[p[0]];
+	min = min_utf_8[l];
 	v = p[0] & ((1 << l) - 1);
 	(*s)++;
 	while (l++ <= 5) {
-		int c = **s - 0x80;
-		if (c < 0 || c >= 0x40) goto ret;
+		unsigned c = **s - 0x80;
+		if (c >= 0x40) {
+			return 0;
+		}
 		(*s)++;
 		v = (v << 6) + c;
 	}
+	if (v < min) {
+		return 0;
+	}
 	return v;
 }
 
@@ -298,15 +306,15 @@ struct conv_table *get_translation_table(int from, int to)
 	static int lfr = -1;
 	static int lto = -1;
 	if (/*from == to ||*/ from == -1 || to == -1) return NULL;
-	if (codepages[to].table == table_utf_8) return get_translation_table_to_utf_8(from);
+	if (to == utf8_table) return get_translation_table_to_utf_8(from);
 	if (table_init) memset(table, 0, sizeof(struct conv_table) * 256), table_init = 0;
 	if (from == lfr && to == lto) return table;
 	lfr = from; lto = to;
 	new_translation_table(table);
-	if (codepages[from].table == table_utf_8) {
+	if (from == utf8_table) {
 		int j;
-		for (j = 0; codepages[to].table[j].c; j++) add_utf_8(table, codepages[to].table[j].u, codepages[to].table[j].u == 0xa0 ? (unsigned char *)"\001" : codepages[to].table[j].u == 0xad ? (unsigned char *)"" : strings[codepages[to].table[j].c]);
-		for (i = 0; unicode_7b[i].x != -1; i++) if (unicode_7b[i].x >= 0x80) add_utf_8(table, unicode_7b[i].x, unicode_7b[i].s);
+		for (j = 0; codepages[to].table[j].c; j++) add_utf_8(table, codepages[to].table[j].u, codepages[to].table[j].u == 0xa0 ? cast_uchar strings[1] : codepages[to].table[j].u == 0xad ? cast_uchar strings[0] : cast_uchar strings[codepages[to].table[j].c]);
+		for (i = 0; unicode_7b[i].x != -1; i++) if (unicode_7b[i].x >= 0x80) add_utf_8(table, unicode_7b[i].x, cast_uchar unicode_7b[i].s);
 	} else for (i = 128; i < 256; i++) {
 		int j;
 		unsigned char *u;
@@ -342,7 +350,7 @@ int get_entity_number(unsigned char *st, int l)
 			if (c >= '0' && c <= '9') n = n * 16 + c - '0';
 			else if (c >= 'A' && c <= 'F') n = n * 16 + c - 'A' + 10;
 			else return -1;
-			if (n >= 0x10000) return -1;
+			if (n > 0x10FFFF) return -1;
 		} while (--l);
 	} else {
 		if (!l) return -1;
@@ -350,7 +358,7 @@ int get_entity_number(unsigned char *st, int l)
 			unsigned char c = *(st++);
 			if (c >= '0' && c <= '9') n = n * 10 + c - '0';
 			else return -1;
-			if (n >= 0x10000) return -1;
+			if (n > 0x10FFFF) return -1;
 		} while (--l);
 	}
 	return n;
@@ -369,7 +377,7 @@ unsigned char *get_entity_string(unsigned char *st, int l, int encoding)
 		while (s <= e) {
 			int c;
 			int m = (s + e) / 2;
-			c = xxstrcmp(entities[m].s, st, l);
+			c = xxstrcmp(cast_uchar entities[m].s, st, l);
 			if (!c) {
 				n = entities[m].c;
 				goto f;
@@ -450,36 +458,9 @@ unsigned char *convert_string(struct conv_table *ct, unsigned char *c, int l, st
 	return buffer;
 }
 
-/* 1 match 0 mismatch */
-static inline int compare_names(unsigned char *one, unsigned char *two)
-{
-
-	while(1){
-		if (!*one) return !*two;
-		if (!*two||(upcase(*one)!=upcase(*two))) return 0;
-		one++;
-		two++;
-	}
-
-}
-
-/*
 int get_cp_index(unsigned char *n)
 {
-	int i, a;
-
-	for (i = 0; codepages[i].name; i++) {
-		if (compare_names(n,codepages[i].name)) return i;
-		for (a = 0; codepages[i].aliases[a]; a++)
-			if (compare_names(n,codepages[i].aliases[a])) return i;
-	}
-	return -1;
-}
-*/
-
-int get_cp_index(unsigned char *n)
-{
-	int i, a, p, q;
+	int i, a, p, q, sl;
 	int ii = -1, ll = 0;
 	for (i = 0; codepages[i].name; i++) {
 		for (a = 0; codepages[i].aliases[a]; a++) {
@@ -488,8 +469,9 @@ int get_cp_index(unsigned char *n)
 					for (q = 1; codepages[i].aliases[a][q]; q++) {
 						if (upcase(n[p+q]) != upcase(codepages[i].aliases[a][q])) goto fail;
 					}
-					if (strlen(codepages[i].aliases[a]) > (size_t)ll) {
-						ll = strlen(codepages[i].aliases[a]);
+					sl = (int)strlen(cast_const_char codepages[i].aliases[a]);
+					if (sl > ll) {
+						ll = sl;
 						ii = i;
 					}
 				}
@@ -502,15 +484,15 @@ int get_cp_index(unsigned char *n)
 
 unsigned char *get_cp_name(int index)
 {
-	if (index < 0) return "none";
-	return codepages[index].name;
+	if (index < 0) return cast_uchar "none";
+	return cast_uchar codepages[index].name;
 }
 
 unsigned char *get_cp_mime_name(int index)
 {
-	if (index < 0) return "none";
+	if (index < 0) return cast_uchar "none";
 	if (!codepages[index].aliases) return NULL;
-	return codepages[index].aliases[0];
+	return cast_uchar codepages[index].aliases[0];
 }
 
 #define UP_EQUAL(a, b) unicode_upcase[a].lo == (b)
@@ -525,7 +507,7 @@ unsigned charset_upcase(unsigned ch, int cp)
 	u = cp2u(ch, cp);
 	BIN_SEARCH(sizeof(unicode_upcase) / sizeof(*unicode_upcase), UP_EQUAL, UP_ABOVE, u, res);
 	if (res == -1) return ch;
-	if (codepages[cp].table == table_utf_8) return unicode_upcase[res].up;
+	if (cp == utf8_table) return unicode_upcase[res].up;
 	str = u2cp(unicode_upcase[res].up, cp, 0);
 	if (!str || !str[0] || str[1]) return ch;
 	return str[0];
@@ -533,16 +515,14 @@ unsigned charset_upcase(unsigned ch, int cp)
 
 unsigned uni_upcase(unsigned ch)
 {
-	int cp = get_cp_index("utf-8");
-	if (cp < 0) cp = 0;
-	return charset_upcase(ch, cp);
+	return charset_upcase(ch, utf8_table);
 }
 
 void charset_upcase_string(unsigned char **chp, int cp)
 {
 	unsigned char *ch = *chp;
 	int i;
-	if (codepages[cp].table == table_utf_8) {
+	if (cp == utf8_table) {
 		ch = unicode_upcase_string(ch);
 		mem_free(*chp);
 		*chp = ch;
@@ -570,8 +550,8 @@ unsigned char *unicode_upcase_string(unsigned char *ch)
 unsigned char *to_utf8_upcase(unsigned char *str, int cp)
 {
 	unsigned char *str1, *str2;
-	struct conv_table *ct = get_translation_table(cp, get_cp_index("utf-8"));
-	str1 = convert_string(ct, str, strlen(str), NULL);
+	struct conv_table *ct = get_translation_table(cp, utf8_table);
+	str1 = convert_string(ct, str, (int)strlen(cast_const_char str), NULL);
 	str2 = unicode_upcase_string(str1);
 	mem_free(str1);
 	return str2;
@@ -584,7 +564,7 @@ int compare_case_utf8(unsigned char *u1, unsigned char *u2)
 	int cc1;
 	while (1) {
 		GET_UTF_8(u2, c2);
-		if (!c2) return u1 - uu1;
+		if (!c2) return (int)(u1 - uu1);
 		skip_discr:
 		GET_UTF_8(u1, c1);
 		BIN_SEARCH(sizeof(unicode_upcase) / sizeof(*unicode_upcase), UP_EQUAL, UP_ABOVE, c1, cc1);
@@ -617,16 +597,16 @@ int strlen_utf8(unsigned char *s)
 
 int cp_len(int cp, unsigned char *s)
 {
-	if (is_cp_special(cp)) return strlen_utf8(s);
-	return strlen((char *)s);
+	if (cp == utf8_table) return strlen_utf8(s);
+	return (int)strlen(cast_const_char s);
 }
 
 unsigned char *cp_strchr(int charset, unsigned char *str, unsigned chr)
 {
-	if (!is_cp_special(charset)) {
+	if (charset != utf8_table) {
 		if (chr >= 0x100)
 			return NULL;
-		return (unsigned char *)strchr(str, chr);
+		return cast_uchar strchr(cast_const_char str, chr);
 	}
 	while (1) {
 		unsigned char *o_str = str;
@@ -636,3 +616,10 @@ unsigned char *cp_strchr(int charset, unsigned char *str, unsigned chr)
 		if (c == chr) return o_str;
 	}
 }
+
+void init_charset(void)
+{
+	utf8_table = get_cp_index(cast_uchar "UTF-8");
+	if (utf8_table == -1) internal("no UTF-8 charset");
+	bookmarks_codepage = utf8_table;
+}
